/*
    Парус 8 - Панели мониторинга - УДП - Доски задач
    Компонент: Фильтр отбора
*/

//---------------------
//Подключение библиотек
//---------------------

import React from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import { Chip, Stack, Icon, IconButton, Box, Menu, MenuItem, Typography } from "@mui/material"; //Интерфейсные компоненты
import { useOrders } from "./hooks.js"; //Хук меню сортировки

//---------
//Константы
//---------

//Стили
const STYLES = {
    ICON_ORDERS: orders => {
        return orders.length > 0 ? { color: "#1976d2" } : {};
    },
    ORDER_MENU: {
        width: "250px"
    },
    ORDER_MENU_ITEM: {
        display: "flex",
        justifyContent: "space-between"
    }
};

//--------------------------
//Вспомогательные компоненты
//--------------------------

//Элемент меню сортировок
const SortMenuItem = ({ item, caption, orders, handleOrderChanged }) => {
    //Кнопка сортировки
    const order = orders.find(o => o.name == item);

    return (
        <MenuItem sx={STYLES.ORDER_MENU_ITEM} key={item} onClick={() => handleOrderChanged(item)}>
            <Typography>{caption}</Typography>
            {order ? order.direction === "ASC" ? <Icon>arrow_upward</Icon> : <Icon>arrow_downward</Icon> : null}
        </MenuItem>
    );
};

//Контроль свойств компонента - Элемент меню сортировок
SortMenuItem.propTypes = {
    item: PropTypes.string.isRequired,
    caption: PropTypes.string.isRequired,
    orders: PropTypes.array,
    handleOrderChanged: PropTypes.func.isRequired
};

//Меню сортировок
const SortMenu = ({ menuOrders, handleOrdersMenuClose, orders, handleOrderChanged }) => {
    return (
        <Menu
            id={`sort_menu`}
            anchorEl={menuOrders.anchorMenuOrders}
            open={menuOrders.openOrders}
            onClose={handleOrdersMenuClose}
            MenuListProps={{ sx: STYLES.ORDER_MENU }}
        >
            <SortMenuItem item={"DPLAN_DATE"} caption={"Дата начала работ"} orders={orders} handleOrderChanged={handleOrderChanged} />
            <SortMenuItem item={"SPREF_NUMB"} caption={"Номер"} orders={orders} handleOrderChanged={handleOrderChanged} />
        </Menu>
    );
};

//Контроль свойств компонента - Меню сортировок
SortMenu.propTypes = {
    menuOrders: PropTypes.object.isRequired,
    handleOrdersMenuClose: PropTypes.func.isRequired,
    orders: PropTypes.array,
    handleOrderChanged: PropTypes.func.isRequired
};

//Элемент фильтра
const FilterItem = ({ caption, value, onClick }) => {
    //При нажатии на элемент
    const handleClick = () => (onClick ? onClick() : null);

    //Генерация содержимого
    return (
        <Chip
            label={
                <Stack direction={"row"} alignItems={"center"}>
                    <strong>{caption}</strong>:&nbsp;{value}
                </Stack>
            }
            variant="outlined"
            onClick={handleClick}
        />
    );
};

//Контроль свойств компонента - Элемент фильтра
FilterItem.propTypes = {
    caption: PropTypes.string.isRequired,
    value: PropTypes.any.isRequired,
    onClick: PropTypes.func
};

//---------------
//Тело компонента
//---------------

//Фильтр отбора
const Filter = ({ filter, selectedDoc, handleFilterClick, handleReload, orders, handleOrderChanged }) => {
    //Меню сортировки
    const [menuOrders, handleOrdersMenuButtonClick, handleOrdersMenuClose] = useOrders();

    //При нажатии на фильтр
    const handleClick = () => (handleFilterClick ? handleFilterClick() : null);

    //Генерация содержимого
    return (
        <Box>
            <Stack direction="row" spacing={1} p={1} alignItems={"center"}>
                <IconButton title="Обновить" onClick={handleReload}>
                    <Icon>refresh</Icon>
                </IconButton>
                <IconButton title="Сортировать" sx={STYLES.ICON_ORDERS(orders)} onClick={handleOrdersMenuButtonClick}>
                    <Icon>sort</Icon>
                </IconButton>
                <IconButton title="Фильтр" onClick={handleClick}>
                    <Icon>filter_alt</Icon>
                </IconButton>
                {filter.type ? <FilterItem caption={"Тип"} value={filter.type} onClick={handleClick} /> : null}
                {filter.sendPerson ? <FilterItem caption={"Исполнитель"} value={filter.sendPerson} onClick={handleClick} /> : null}
                {filter.sendDivision ? <FilterItem caption={"Подразделение"} value={filter.sendDivision} onClick={handleClick} /> : null}
                {filter.sendUsrGrp ? <FilterItem caption={"Группа пользователей"} value={filter.sendUsrGrp} onClick={handleClick} /> : null}
                {filter.docLink && selectedDoc ? <FilterItem caption={"Учётный документ"} value={selectedDoc.descr} onClick={handleClick} /> : null}
            </Stack>
            <SortMenu menuOrders={menuOrders} handleOrdersMenuClose={handleOrdersMenuClose} orders={orders} handleOrderChanged={handleOrderChanged} />
        </Box>
    );
};

//Контроль свойств компонента - Фильтр отбора
Filter.propTypes = {
    filter: PropTypes.object.isRequired,
    selectedDoc: PropTypes.object,
    handleFilterClick: PropTypes.func,
    handleReload: PropTypes.func,
    orders: PropTypes.array,
    handleOrderChanged: PropTypes.func
};

//--------------------
//Интерфейс компонента
//--------------------

export { Filter };
