/*
    Парус 8 - Панели мониторинга - УДП - Доски задач
    Компонент: Диалоговое окно фильтра отбора
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useState, useContext } from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import { Dialog, DialogTitle, IconButton, Icon, DialogContent, DialogActions, Button, Box } from "@mui/material"; //Интерфейсные компоненты
import { FilterInputField } from "./filter_input_field"; //Компонент поля ввода
import { ApplicationСtx } from "../../../context/application"; //Контекст приложения

//---------
//Константы
//---------

//Стили
const STYLES = {
    DIALOG_ACTIONS: { justifyContent: "center" },
    CLOSE_BUTTON: {
        position: "absolute",
        right: 8,
        top: 8,
        color: theme => theme.palette.grey[500]
    }
};

//-----------------------
//Вспомогательные функции
//-----------------------

//Выбор типа события
const selectEventType = (showDictionary, callBack) => {
    showDictionary({
        unitCode: "ClientEventTypes",
        callBack: res => (res.success === true ? callBack(res.outParameters.out_EVNTYPE_CODE) : callBack(null))
    });
};

//Выбор производственного объекта
const selectSendPerson = (showDictionary, callBack) => {
    showDictionary({
        unitCode: "ClientPersons",
        callBack: res => (res.success === true ? callBack(res.outParameters.out_CODE) : callBack(null))
    });
};

//Выбор подразделения
const selectSendDivision = (showDictionary, callBack) => {
    showDictionary({
        unitCode: "INS_DEPARTMENT",
        callBack: res => (res.success === true ? callBack(res.outParameters.out_CODE) : callBack(null))
    });
};

//Выбор группы пользователей
const selectSendUsrGrp = (showDictionary, callBack) => {
    showDictionary({
        unitCode: "CostStaffGroups",
        callBack: res => (res.success === true ? callBack(res.outParameters.out_CODE) : callBack(null))
    });
};

//---------------
//Тело компонента
//---------------

//Диалоговое окно фильтра отбора
const FilterDialog = ({ initial, onCancel, onOk }) => {
    //Собственное состояние
    const [filter, setFilter] = useState({ ...initial });

    //Подключение к контексту приложения
    const { pOnlineShowDictionary } = useContext(ApplicationСtx);

    //При закрытии диалога без изменения фильтра
    const handleCancel = () => (onCancel ? onCancel() : null);

    //При очистке фильтра
    const handleClear = () => {
        setFilter({
            type: "",
            sendPerson: "",
            sendDivision: "",
            sendUsrGrp: ""
        });
    };

    //При закрытии диалога с изменением фильтра
    const handleOK = () => (onOk ? onOk(filter) : null);

    //При изменении значения элемента
    const handleFilterItemChange = (item, value) => setFilter(pv => ({ ...pv, [item]: value }));

    //Генерация содержимого
    return (
        <div>
            <Dialog open onClose={handleCancel} fullWidth maxWidth="sm">
                <DialogTitle>Фильтр отбора</DialogTitle>
                <IconButton aria-label="close" onClick={handleCancel} sx={STYLES.CLOSE_BUTTON}>
                    <Icon>close</Icon>
                </IconButton>
                <DialogContent>
                    <Box component="section" p={1}>
                        <FilterInputField
                            elementCode="type"
                            elementValue={filter.type}
                            labelText="Тип"
                            dictionary={callBack => selectEventType(pOnlineShowDictionary, callBack)}
                            onChange={handleFilterItemChange}
                        />
                    </Box>
                    <Box component="section" p={1}>
                        <FilterInputField
                            elementCode="sendPerson"
                            elementValue={filter.sendPerson}
                            labelText="Исполнитель"
                            dictionary={callBack => selectSendPerson(pOnlineShowDictionary, callBack)}
                            onChange={handleFilterItemChange}
                        />
                    </Box>
                    <Box component="section" p={1}>
                        <FilterInputField
                            elementCode="sendDivision"
                            elementValue={filter.sendDivision}
                            labelText="Подразделение"
                            dictionary={callBack => selectSendDivision(pOnlineShowDictionary, callBack)}
                            onChange={handleFilterItemChange}
                        />
                    </Box>
                    <Box component="section" p={1}>
                        <FilterInputField
                            elementCode="sendUsrGrp"
                            elementValue={filter.sendUsrGrp}
                            labelText="Группа пользователей"
                            dictionary={callBack => selectSendUsrGrp(pOnlineShowDictionary, callBack)}
                            onChange={handleFilterItemChange}
                        />
                    </Box>
                </DialogContent>
                <DialogActions sx={STYLES.DIALOG_ACTIONS}>
                    <Button variant="text" onClick={handleOK}>
                        Применить
                    </Button>
                    <Button variant="text" onClick={handleClear}>
                        Очистить
                    </Button>
                    <Button variant="text" onClick={handleCancel}>
                        Отмена
                    </Button>
                </DialogActions>
            </Dialog>
        </div>
    );
};

//Контроль свойств компонента - Диалоговое окно фильтра отбора
FilterDialog.propTypes = {
    initial: PropTypes.object.isRequired,
    onOk: PropTypes.func,
    onCancel: PropTypes.func
};

//--------------------
//Интерфейс компонента
//--------------------

export { FilterDialog };
