/*
    Парус 8 - Панели мониторинга - РО - Редактор настройки регламентированного отчёта
    Панель мониторинга: Корневая панель редактора
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useCallback, useContext, useState, useEffect } from "react"; //Классы React
import { Box, Tab, Tabs, IconButton, Icon, Stack, Button } from "@mui/material"; //Интерфейсные компоненты
import { P8PDataGrid, P8P_DATA_GRID_SIZE } from "../../components/p8p_data_grid"; //Таблица данных
import { P8P_DATA_GRID_CONFIG_PROPS } from "../../config_wrapper"; //Подключение компонентов к настройкам приложения
import { ApplicationСtx } from "../../context/application"; //Контекст приложения
import { BackEndСtx } from "../../context/backend"; //Контекст взаимодействия с сервером
import { NavigationCtx } from "../../context/navigation"; //Контекст навигации
import { MessagingСtx } from "../../context/messaging"; //Контекст сообщений
import { SectionTabPanel } from "./section_tab_panel"; //Компонент вкладки раздела
import { IUDFormDialog } from "./iud_form_dialog"; //Диалог добавления/исправления/удаления компонентов настройки регламентированного отчёта
import { confSctnMrkCellRender } from "./layouts"; //Дополнительная разметка и вёрстка клиентских элементов
import { STATUSES } from "./iud_form_dialog"; //Статусы диалогового окна
import { TEXTS } from "../../../app.text"; //Текстовые константы
import { STYLES as COMMON_STYLES } from "./layouts"; //Общие стили
import { useWindowResize } from "./hooks"; //Пользовательские хуки

//---------
//Константы
//---------

//Высота меню Парус (пиксели)
const pxOuterMenuH = 53;
//Высота заголовка панели (пиксели)
const pxPanelHeaderH = 64;
//Ширина кнопки добавления раздела (пиксели)
const pxSectionAddButtonW = 40;

//Стили
const STYLES = {
    CONTAINER: { width: "100%" },
    PANELS_MAIN_COLOR: { backgroundColor: "#1976d2" },
    ICON_WHITE: { color: "white" },
    TABS_BOTTOM_LINE: { borderBottom: 1, borderColor: "divider" },
    TABS_PADDING: { padding: "5px" },
    TABS_SIZES: (width, pxSectionAddButtonW) => ({ maxHeight: 150, maxWidth: width - pxSectionAddButtonW }),
    GRID_PADDING: { paddingTop: 1, paddingBottom: 1 },
    GRID_SIZES: (width, height, pxOuterMenuH, pxPanelHeaderH, pxTabsH) => ({
        padding: 0,
        minWidth: width * 0.95,
        minHeight: (height - pxOuterMenuH - pxPanelHeaderH - pxTabsH) * 0.88,
        maxWidth: width * 0.95,
        maxHeight: (height - pxOuterMenuH - pxPanelHeaderH - pxTabsH) * 0.88
    })
};

//-----------
//Тело модуля
//-----------

//Редактор настройки регламентированного отчёта
const RrpConfEditor = () => {
    const dataGrid = {
        rn: 0,
        code: "",
        name: "",
        dataLoaded: false,
        columnsDef: [],
        groups: [],
        rows: [],
        fixedHeader: false,
        fixedColumns: 0,
        reload: false
    };

    //Собственное состояние
    const [rrpDoc, setRrpDoc] = useState({
        docLoaded: false,
        sections: [],
        reload: true
    });

    //Состояние массива данных разделов
    const [dataGrids] = useState([]);

    //Состояние раздела
    const [tabValue, setTabValue] = useState("");

    //Состояние открытия диалогового окна
    const [formOpen, setForm] = useState(false);

    //Состояние диалогового окна
    const [formData, setFormData] = useState({
        rn: "",
        prn: "",
        sctnName: "",
        sctnCode: "",
        status: "",
        code: "",
        name: "",
        colName: "",
        colCode: "",
        colVCode: "",
        colVRn: 0,
        rowName: "",
        rowCode: "",
        rowVCode: "",
        rowVRn: 0
    });

    //Открытие диалогового окна
    const openForm = () => {
        setForm(true);
    };

    //Очистка диалогового окна
    const clearFormData = () => {
        setFormData({
            rn: "",
            prn: "",
            sctnName: "",
            sctnCode: "",
            status: "",
            code: "",
            name: "",
            colName: "",
            colCode: "",
            colVCode: "",
            colVRn: 0,
            rowName: "",
            rowCode: "",
            rowVCode: "",
            rowVRn: 0
        });
    };

    //Подключение к контексту взаимодействия с сервером
    const { executeStored } = useContext(BackEndСtx);

    //Подключение к контексту приложения
    const { pOnlineShowUnit } = useContext(ApplicationСtx);

    //Подключение к контексту навигации
    const { getNavigationSearch } = useContext(NavigationCtx);

    //Подключение к контексту сообщений
    const { showMsgErr } = useContext(MessagingСtx);

    //Переключение раздела
    const handleSectionChange = (event, newValue) => {
        setTabValue(newValue);
    };

    //Отработка нажатия на кнопку добавления секции
    const addSectionClick = () => {
        setFormData({ status: STATUSES.CREATE, prn: Number(getNavigationSearch().NRN) });
        openForm();
    };

    //Отработка нажатия на кнопку исправления секции
    const editSectionClick = (rn, code, name) => {
        setFormData({ rn: rn, code: code, name: name, status: STATUSES.EDIT });
        openForm();
    };

    //Отработка нажатия на кнопку удаления секции
    const deleteSectionClick = (rn, code, name) => {
        setFormData({ rn: rn, code: code, name: name, status: STATUSES.DELETE });
        openForm();
    };

    //Отработка нажатия на кнопку добавления показателя раздела
    const addSectionMarkClick = (prn, sctnCode, sctnName) => {
        setFormData({ status: STATUSES.RRPCONFSCTNMRK_CREATE, prn: prn, sctnCode: sctnCode, sctnName: sctnName });
        openForm();
    };

    //Отработка нажатия на кнопку исправления показателя раздела
    const editSectionMarkClick = (rn, name) => {
        setFormData({ status: STATUSES.RRPCONFSCTNMRK_EDIT, rn: rn, name: name });
        openForm();
    };

    //Отработка нажатия на кнопку удаления показателя раздела
    const deleteSectionMarkClick = (rn, name) => {
        setFormData({ status: STATUSES.RRPCONFSCTNMRK_DELETE, rn: rn, name: name });
        openForm();
    };

    //Отображение показателя раздела
    const showSectionMark = async rn => {
        const data = await executeStored({
            stored: "PKG_P8PANELS_RRPCONFED.RRPCONFSCTNMRK_GET_CODES",
            args: {
                NRN: rn
            },
            tagValueProcessor: () => undefined
        });
        if (data) {
            pOnlineShowUnit({
                unitCode: "RRPConfig",
                showMethod: "main_mrk_settings",
                inputParameters: [
                    { name: "in_CODE", value: data.SRRPCONF },
                    { name: "in_SCTN_CODE", value: data.SRRPCONFSCTN },
                    { name: "in_MRK_CODE", value: data.SRRPCONFSCTNMRK }
                ]
            });
        } else showMsgErr(TEXTS.NO_DATA_FOUND);
    };

    //Формирование разделов
    const a11yProps = index => {
        return {
            id: `simple-tab-${index}`,
            "aria-controls": `simple-tabpanel-${index}`
        };
    };

    //Загрузка данных разделов регламентированного отчёта
    const loadData = useCallback(async () => {
        if (rrpDoc.reload) {
            //Переменная номера раздела с фокусом
            let tabFocus = 0;
            const data = await executeStored({
                stored: "PKG_P8PANELS_RRPCONFED.RRPCONF_GET_SECTIONS",
                args: {
                    NRN_RRPCONF: Number(getNavigationSearch().NRN)
                },
                respArg: "COUT"
            });
            //Флаг первой загрузки данных
            let firstLoad = dataGrids.length == 0 ? true : false;
            //Копирование массива уже загруженных разделов
            let cloneDGs = dataGrids.slice();
            //Массив из нескольких разделов и из одного
            const sections = data.SECTIONS ? (data.SECTIONS.length ? data.SECTIONS : [data.SECTIONS]) : [];
            //Заполнение очередного раздела по шаблону
            sections.map(s => {
                let dg = {};
                Object.assign(dg, dataGrid, {
                    rn: s.NRN,
                    code: s.SCODE,
                    name: s.SNAME,
                    dataLoaded: true,
                    columnsDef: [...(s.XDATA.XCOLUMNS_DEF || [])],
                    groups: [...(s.XDATA.XGROUPS || [])],
                    rows: [...(s.XDATA.XROWS || [])],
                    fixedHeader: s.XDATA.XDATA_GRID.fixedHeader,
                    fixedColumns: s.XDATA.XDATA_GRID.fixedColumns,
                    reload: false
                });
                //Ищем загружен ли уже раздел с таким же ид.
                const dgItem = dataGrids.find(x => x.rn === dg.rn);
                //Его индекс, если нет соответствия, то -1
                let index = dataGrids.indexOf(dgItem);
                //Если было соответствие
                if (dgItem) {
                    //Если в нём не найдено изменений
                    if (JSON.stringify(dgItem, null, 4) === JSON.stringify(dg, null, 4)) {
                        //То из копированного массива его удаляем
                        cloneDGs.splice(cloneDGs.indexOf(cloneDGs.find(x => x.rn === dgItem.rn)), 1);
                    } else {
                        //Иначе обновляем раздел в массиве
                        dataGrids[index] = dg;
                        //Удаляем из копированного массива
                        cloneDGs.splice(cloneDGs.indexOf(cloneDGs.find(x => x.rn === dg.rn)), 1);
                        //Устанавливаем фокус на обновлённый раздел
                        tabFocus = index;
                    }
                } else {
                    //Если раздел новый, то добавляем его в массив данных
                    dataGrids.push(dg);
                    //И устанавливаем на него фокус, если флаг первой загрузки = false
                    tabFocus = !firstLoad ? dataGrids.length - 1 : 0;
                }
            });
            //Обходим разделы, что остались в копированном массиве (на удаление)
            cloneDGs.map(s => {
                let curIndex = dataGrids.indexOf(dataGrids.find(x => x.rn === s.rn));
                //Устаревший раздел удаляем из массива данных
                dataGrids.splice(curIndex, 1);
                //Фокус на предшествующий раздел
                if (curIndex > 0) tabFocus = curIndex - 1;
                //Иначе фокус на следующий, если был удалён первый раздел
                else tabFocus = curIndex;
            });
            setRrpDoc(pv => ({
                ...pv,
                docLoaded: true,
                reload: false,
                sections: dataGrids
            }));
            setTabValue(tabFocus);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [rrpDoc.reload, rrpDoc.docLoaded, dataGrid.reload, dataGrid.docLoaded, executeStored]);

    //При необходимости обновить данные таблицы
    useEffect(() => {
        loadData();
    }, [rrpDoc.reload, dataGrid.reload, loadData]);

    //При изменениях элемента
    const handleDialogReload = () => {
        setRrpDoc(pv => ({ ...pv, reload: true }));
    };

    //При закрытии диалога
    const handleDialogClose = () => {
        setForm(false);
        clearFormData();
    };

    //Состояние ширины и высоты рабочей области окна
    const [width, height] = useWindowResize();

    //Состояние высоты вкладок с разделами
    const [pxTabsH, setPxTabsH] = useState(0);

    //При рендере данных
    useEffect(() => {
        rrpDoc.docLoaded ? setPxTabsH(document.getElementById("sectionTabs").offsetHeight) : null;
    }, [rrpDoc.docLoaded]);

    //Генерация содержимого
    return (
        <Box sx={STYLES.CONTAINER}>
            {formOpen ? <IUDFormDialog initial={formData} onClose={handleDialogClose} onReload={handleDialogReload} /> : null}
            {rrpDoc.docLoaded ? (
                <Box>
                    <Stack direction="row" sx={STYLES.TABS_BOTTOM_LINE}>
                        <Tabs
                            id="sectionTabs"
                            value={tabValue}
                            onChange={handleSectionChange}
                            variant="scrollable"
                            scrollButtons={false}
                            visibleScrollbar
                            aria-label="section tab"
                            sx={STYLES.TABS_SIZES(width, pxSectionAddButtonW)}
                        >
                            {rrpDoc.sections.map((s, i) => {
                                return (
                                    <Tab
                                        key={s.rn}
                                        {...a11yProps(i)}
                                        sx={STYLES.TABS_PADDING}
                                        label={
                                            <Box sx={COMMON_STYLES.BOX_ROW}>
                                                {s.name}
                                                <IconButton component="span" onClick={() => editSectionClick(s.rn, s.code, s.name)}>
                                                    <Icon>edit</Icon>
                                                </IconButton>
                                                <IconButton component="span" onClick={() => deleteSectionClick(s.rn, s.code, s.name)}>
                                                    <Icon>delete</Icon>
                                                </IconButton>
                                            </Box>
                                        }
                                        wrapped
                                    />
                                );
                            })}
                        </Tabs>
                        <Box display="flex" justifyContent="center" alignItems="center" sx={STYLES.PANELS_MAIN_COLOR}>
                            <IconButton onClick={addSectionClick}>
                                <Icon sx={STYLES.ICON_WHITE}>add</Icon>
                            </IconButton>
                        </Box>
                    </Stack>
                    {rrpDoc.sections.map((s, i) => {
                        return (
                            <SectionTabPanel key={s.rn} value={tabValue} index={i}>
                                <Button onClick={() => addSectionMarkClick(s.rn, s.code, s.name)}>Добавить</Button>
                                {s.dataLoaded && s.columnsDef.length > 1 ? (
                                    <Box sx={{ ...STYLES.GRID_PADDING, ...COMMON_STYLES.BOX_ROW }}>
                                        <P8PDataGrid
                                            {...P8P_DATA_GRID_CONFIG_PROPS}
                                            containerComponentProps={{
                                                elevation: 6,
                                                style: STYLES.GRID_SIZES(width, height, pxOuterMenuH, pxPanelHeaderH, pxTabsH)
                                            }}
                                            columnsDef={s.columnsDef}
                                            groups={s.groups}
                                            rows={s.rows}
                                            fixedHeader={s.fixedHeader}
                                            fixedColumns={s.fixedColumns}
                                            size={P8P_DATA_GRID_SIZE.LARGE}
                                            reloading={s.reload}
                                            dataCellRender={prms =>
                                                confSctnMrkCellRender({
                                                    ...prms,
                                                    onLinkClick: showSectionMark,
                                                    onEditClick: editSectionMarkClick,
                                                    onDeleteClick: deleteSectionMarkClick
                                                })
                                            }
                                        />
                                    </Box>
                                ) : null}
                            </SectionTabPanel>
                        );
                    })}
                </Box>
            ) : null}
        </Box>
    );
};

//----------------
//Интерфейс модуля
//----------------

export { RrpConfEditor };
